@@ -835,8 +835,8 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 			case AF_INET:
 				((struct sockaddr_in *)sa)->sin_port = htons(port);
 				socklen = sizeof(struct sockaddr_in);
-				if (bindto && strchr(bindto, ':')) {
-					/* IPV4 sock cannot bind to IPV6 address */
+				if (bindto && (strchr(bindto, ':') || !strcmp(bindto, "0"))) {
+					/* IPV4 sock can not bind to IPV6 address */
 					bindto = NULL;
 				}
 				break;
@@ -861,7 +861,7 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 #if HAVE_IPV6 && HAVE_INET_PTON
 				struct sockaddr_in6 in6;
 #endif
-			} local_address;
+			} local_address = {0};
 			int local_address_len = 0;
 
 			if (sa->sa_family == AF_INET) {
@@ -873,7 +873,6 @@ php_socket_t php_network_connect_socket_to_host(const char *host, unsigned short
 					local_address_len = sizeof(struct sockaddr_in);
 					local_address.in4.sin_family = sa->sa_family;
 					local_address.in4.sin_port = htons(bindport);
-					memset(&(local_address.in4.sin_zero), 0, sizeof(local_address.in4.sin_zero));
 				}
 			}
 #if HAVE_IPV6 && HAVE_INET_PTON
